from .scraper import Scraper
import io
import logging
import os
import string

FILEPATH_CHARS = '/-_.() %s%s' % (string.ascii_letters, string.digits)
log = logging.getLogger(__name__)


class Crawler(object):
    def __init__(self, start, out='_crawled/', max_depth=3,
                 force_prefix=None, scraper=None):
        start = self.absolute_path(start)
        self.urls = [(start, 0)]
        self.done = set()
        self.out = out
        self.max_depth = max_depth
        self.force_prefix = force_prefix or self.guess_prefix(start)
        self.scraper = scraper or Scraper()

    def absolute_path(self, path):
        if '://' in path:
            return path
        if os.path.isfile(path):
            return 'file://{}'.format(os.path.abspath(path))
        return path

    def guess_prefix(self, path):
        if path.startswith('http'):
            path = self.complete_url(path)
        base, sep, _ = path.rpartition('/')
        return base + sep

    def complete_url(self, url):
        if url.endswith('/'):
            return url + 'index.html'
        if url[-1] in ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9'):
            return url + '/index.html'
        if '.' not in url.rpartition('/')[2]:
            return url + '/index.html'
        return url

    def crawl(self):
        count = 0
        while self.urls:
            url, depth = self.urls.pop(0)
            url = self.complete_url(url)
            if url in self.done:
                continue
            if not url.startswith(self.force_prefix):
                continue

            log.debug('Crawl url: {}'.format(url))
            html = self.scraper.html(url)
            if not html:
                log.warn('Invalid {}. Skipping.'.format(url))
                continue

            filename = ''.join(c
                               for c in url[len(self.force_prefix):]
                               if c in FILEPATH_CHARS)
            path = os.path.abspath(os.path.join(self.out, filename))
            if not path.startswith(os.path.abspath(self.out)):
                log.warn('Path {} is outside of {}. Skipping.'
                         ''.format(path, self.out))
                continue
            log.debug('Writing file: {}'.format(path))
            dirname = os.path.dirname(path)
            if not os.path.isdir(dirname):
                os.makedirs(dirname)
            with io.open(path, 'w') as f:
                f.write(html)
            self.done.add(url)
            count += 1

            if depth >= self.max_depth:
                continue

            self.urls += [(lurl, depth + 1)
                          for lurl in self.scraper.link_urls(url)]

        return count



"""Crawl and render JavaScript templates."""

from .crawler import Crawler
import argparse
import logging


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('--start', default='_build/index.html',
                        help='start url or file')
    parser.add_argument('-o', '--out', default='_crawled/',
                        help='output directory')
    parser.add_argument('--depth', default=3, type=int,
                        help='depth of the crawl')
    args = parser.parse_args()

    logging.basicConfig(level=logging.WARNING)
    Crawler(args.start, args.out, max_depth=args.depth).crawl()



from setuptools import setup

# extract version from __init__.py
with open('localcrawl/__init__.py', 'r') as f:
    version_line = [l for l in f if l.startswith('__version__')][0]
    VERSION = version_line.split('=')[1].strip()[1:-1]


setup(
    name='localcrawl',
    version=VERSION,
    packages=['localcrawl'],
    license='MIT',
    description='Crawl rendered JavaScript templates from a local server.',
    long_description=open('README.rst').read(),
    author='Sven Kreiss',
    author_email='me@svenkreiss.com',
    url='https://github.com/svenkreiss/localcrawl',

    install_requires=[
        'selenium>=2.53.6',
    ],
    extras_require={
        'tests': [
            'flake8==2.5.4',
            'hacking>=0.11.0',
            'nose>=1.3.4',
        ]
    },
    entry_points={
        'console_scripts': ['localcrawl=localcrawl.cli:main'],
    },

    tests_require=[
        'nose>=1.3.4',
    ],
    test_suite='nose.collector',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: PyPy',
    ]
)



import selenium.webdriver
import time


class Scraper(object):
    def __init__(self, driver=None, sleep=0.1):
        self.driver = driver or selenium.webdriver.PhantomJS()
        self.sleep = sleep
        self._current_url = None
        self._invalid = True

    def _get(self, url):
        if url == self._current_url:
            return

        self._current_url = url
        old_source = self.driver.page_source
        self.driver.get(url)
        time.sleep(self.sleep)
        self._invalid = old_source == self.driver.page_source

    def html(self, url):
        self._get(url)
        if self._invalid:
            return ''
        return self.driver.page_source

    def link_urls(self, url):
        self._get(url)
        if self._invalid:
            return []
        links = self.driver.find_elements_by_tag_name('a')
        return [l.get_attribute('href') for l in links]
